@@ -23,36 +23,35 @@
 # delete this exception statement from your version.
 
 from itertools import chain
-import logging
 import re
 
 from gi.repository import Gio, GLib
 
-from gnomemusic import log
 from gnomemusic.albumartcache import lookup_art_file_from_cache
-from gnomemusic.widgets.songwidget import SongWidget
 from gnomemusic.gstplayer import Playback
 from gnomemusic.player import PlayerPlaylist, RepeatMode
-
-logger = logging.getLogger(__name__)
+from gnomemusic.widgets.songwidget import SongWidget
 
 
 class DBusInterface:
 
-    def __init__(self, name, path):
+    def __init__(self, name, path, application):
         """Etablish a D-Bus session connection
 
         :param str name: interface name
         :param str path: object path
+        :param GtkApplication application: The Application object
         """
+        self._log = application.props.log
         self._path = path
+        self._signals = None
         Gio.bus_get(Gio.BusType.SESSION, None, self._bus_get_sync, name)
 
     def _bus_get_sync(self, source, res, name):
         try:
             self._con = Gio.bus_get_finish(res)
         except GLib.Error as e:
-            logger.warning(
+            self._log.warning(
                 "Unable to connect to to session bus: {}".format(e.message))
             return
 
@@ -124,6 +123,9 @@ class DBusInterface:
             invocation.return_value(None)
 
     def _dbus_emit_signal(self, signal_name, values):
+        if self._signals is None:
+            return
+
         signal = self._signals[signal_name]
         parameters = []
         for arg_name, arg_signature in signal['args'].items():
@@ -274,15 +276,13 @@ class MPRIS(DBusInterface):
 
     _playlist_nb_songs = 10
 
-    def __repr__(self):
-        return "<MPRIS>"
-
     def __init__(self, app):
         name = 'org.mpris.MediaPlayer2.GnomeMusic'
         path = '/org/mpris/MediaPlayer2'
-        super().__init__(name, path)
+        super().__init__(name, path, app)
 
         self._app = app
+        self._log = app.props.log
         self._player = app.props.player
         self._player.connect(
             'song-changed', self._on_current_song_changed)
@@ -299,8 +299,11 @@ class MPRIS(DBusInterface):
             "playlist-loaded", self._on_player_playlist_changed)
 
         self._playlists_model = self._coremodel.props.playlists_sort
-        self._playlists_loaded_id = self._coremodel.connect(
-            "playlists-loaded", self._on_playlists_loaded)
+        n_items = self._playlists_model.get_n_items()
+        self._on_playlists_items_changed(
+            self._playlists_model, 0, n_items, n_items)
+        self._playlists_model.connect(
+            "items-changed", self._on_playlists_items_changed)
 
         self._player_playlist_type = None
         self._path_list = []
@@ -312,9 +315,7 @@ class MPRIS(DBusInterface):
         self._previous_loop_status = ""
         self._previous_mpris_playlist = self._get_active_playlist()
         self._previous_playback_status = "Stopped"
-        self._previous_playlist_count = 0
 
-    @log
     def _get_playback_status(self):
         state = self._player.props.state
         if state == Playback.STOPPED:
@@ -324,7 +325,6 @@ class MPRIS(DBusInterface):
         else:
             return 'Playing'
 
-    @log
     def _get_loop_status(self):
         if self._player.props.repeat_mode == RepeatMode.ALL:
             return "Playlist"
@@ -333,7 +333,6 @@ class MPRIS(DBusInterface):
         else:
             return "None"
 
-    @log
     def _get_metadata(self, coresong=None, index=None):
         song_dbus_path = self._get_song_dbus_path(coresong, index)
         if not self._player.props.current_song:
@@ -392,7 +391,6 @@ class MPRIS(DBusInterface):
 
         return metadata
 
-    @log
     def _get_song_dbus_path(self, coresong=None, index=None):
         """Convert a Grilo media to a D-Bus path
 
@@ -419,7 +417,6 @@ class MPRIS(DBusInterface):
             id_hex, index)
         return path
 
-    @log
     def _update_tracklist(self):
         previous_path_list = self._path_list
         self._path_list = []
@@ -466,7 +463,6 @@ class MPRIS(DBusInterface):
             current_song_path = self._get_song_dbus_path()
             self._track_list_replaced(self._path_list, current_song_path)
 
-    @log
     def _get_playlist_dbus_path(self, playlist):
         """Convert a playlist to a D-Bus path
 
@@ -481,13 +477,11 @@ class MPRIS(DBusInterface):
         pl_id = playlist.props.pl_id or playlist.props.tag_text
         return "/org/gnome/GnomeMusic/Playlist/{}".format(pl_id)
 
-    @log
     def _get_mpris_playlist_from_playlist(self, playlist):
         playlist_name = playlist.props.title
         path = self._get_playlist_dbus_path(playlist)
         return (path, playlist_name, "")
 
-    @log
     def _get_active_playlist(self):
         """Get Active Maybe_Playlist
 
@@ -508,7 +502,6 @@ class MPRIS(DBusInterface):
             current_playlist)
         return (True, mpris_playlist)
 
-    @log
     def _on_current_song_changed(self, player):
         # In repeat song mode, no metadata has changed if the
         # player was already started
@@ -552,7 +545,6 @@ class MPRIS(DBusInterface):
         self._properties_changed(
             MPRIS.MEDIA_PLAYER2_PLAYER_IFACE, properties, [])
 
-    @log
     def _on_player_state_changed(self, klass, args):
         playback_status = self._get_playback_status()
         if playback_status == self._previous_playback_status:
@@ -563,7 +555,6 @@ class MPRIS(DBusInterface):
             MPRIS.MEDIA_PLAYER2_PLAYER_IFACE,
             {'PlaybackStatus': GLib.Variant('s', playback_status), }, [])
 
-    @log
     def _on_repeat_mode_changed(self, player, param):
         self._update_tracklist()
 
@@ -584,12 +575,10 @@ class MPRIS(DBusInterface):
         self._properties_changed(
             MPRIS.MEDIA_PLAYER2_PLAYER_IFACE, properties, [])
 
-    @log
     def _on_seek_finished(self, player):
         position_second = self._player.get_position()
         self._seeked(int(position_second * 1e6))
 
-    @log
     def _on_player_playlist_changed(self, coremodel, playlist_type):
         self._player_playlist_type = playlist_type
 
@@ -603,34 +592,19 @@ class MPRIS(DBusInterface):
         self._properties_changed(
             MPRIS.MEDIA_PLAYER2_PLAYLISTS_IFACE, properties, [])
 
-    def _on_playlists_loaded(self, klass):
-        self._coremodel.disconnect(self._playlists_loaded_id)
-        for playlist in self._playlists_model:
-            playlist.connect("notify::title", self._on_playlist_renamed)
-            playlist.connect(
-                "notify::active", self._on_player_playlist_changed)
+    def _on_playlists_items_changed(self, model, position, removed, added):
+        if added > 0:
+            for i in range(added):
+                playlist = model[position + i]
+                playlist.connect("notify::title", self._on_playlist_renamed)
+                playlist.connect(
+                    "notify::active", self._on_player_playlist_changed)
 
-        self._playlists_model.connect(
-            "items-changed", self._on_playlists_count_changed)
-        self._on_playlists_count_changed(None, None, 0, 0)
-
-    @log
-    def _on_playlists_count_changed(self, model, position, removed, added):
-        playlist_count = self._playlists_model.get_n_items()
-        if playlist_count == self._previous_playlist_count:
-            return
-
-        self._previous_playlist_count = playlist_count
+        playlist_count = model.get_n_items()
         properties = {"PlaylistCount": GLib.Variant("u", playlist_count)}
         self._properties_changed(
             MPRIS.MEDIA_PLAYER2_PLAYLISTS_IFACE, properties, [])
 
-        if added == 0:
-            return
-
-        model[position].connect("notify::title", self._on_playlist_renamed)
-
-    @log
     def _on_playlist_renamed(self, playlist, param):
         mpris_playlist = self._get_mpris_playlist_from_playlist(playlist)
         self._dbus_emit_signal('PlaylistChanged', {'Playlist': mpris_playlist})
@@ -769,6 +743,17 @@ class MPRIS(DBusInterface):
             "CurrentTrack": self._get_song_dbus_path()}
         self._dbus_emit_signal("TrackListReplaced", parameters)
 
+    def _load_player_playlist(self, playlist):
+        def _on_playlist_loaded(klass, playlist_type):
+            self._player.play()
+            self._coremodel.disconnect(loaded_id)
+
+        loaded_id = self._coremodel.connect(
+            "playlist-loaded", _on_playlist_loaded)
+        self._coremodel.props.active_playlist = playlist
+        self._coremodel.set_player_model(
+            PlayerPlaylist.Type.PLAYLIST, playlist.props.model)
+
     def _activate_playlist(self, playlist_path):
         """Starts playing the given playlist (MPRIS Method).
 
@@ -780,8 +765,18 @@ class MPRIS(DBusInterface):
                 selected_playlist = playlist
                 break
 
-        if selected_playlist is not None:
-            self._coremodel.props.active_playlist = selected_playlist
+        if selected_playlist is None:
+            return
+
+        def _on_playlist_model_loaded(playlist):
+            playlist.disconnect(signal_id)
+            self._load_player_playlist(playlist)
+
+        if selected_playlist.props.model.get_n_items() > 0:
+            self._load_player_playlist(selected_playlist)
+        else:
+            signal_id = selected_playlist.connect(
+                "playlist-loaded", _on_playlist_model_loaded)
 
     def _get_playlists(self, index, max_count, order, reverse):
         """Gets a set of playlists (MPRIS Method).
@@ -816,7 +811,7 @@ class MPRIS(DBusInterface):
         except KeyError:
             msg = "MPRIS does not handle {} property from {} interface".format(
                 property_name, interface_name)
-            logger.warning(msg)
+            self._log.warning(msg)
             raise ValueError(msg)
 
     def _get_all(self, interface_name):
@@ -880,7 +875,7 @@ class MPRIS(DBusInterface):
         elif interface_name == 'org.freedesktop.DBus.Introspectable':
             return {}
         else:
-            logger.warning(
+            self._log.warning(
                 "MPRIS does not implement {} interface".format(interface_name))
 
     def _set(self, interface_name, property_name, new_value):
@@ -903,7 +898,7 @@ class MPRIS(DBusInterface):
                 else:
                     self._player.props.repeat_mode = RepeatMode.NONE
         else:
-            logger.warning(
+            self._log.warning(
                 "MPRIS does not implement {} interface".format(interface_name))
 
     def _properties_changed(self, interface_name, changed_properties,
